<?php
/**
 * Created by PhpStorm.
 * User: Noa
 * Date: 10/10/2017
 * Time: 13:28
 */

namespace Retailmenot\Lab\Middleware\middlewares;


use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;

class RmnMiddleware
{
    /**
     * @param RequestInterface $request
     * @param ResponseInterface $response
     * @param callable|null $next
     * @return ResponseInterface|static
     */
    public function __invoke(RequestInterface $request, ResponseInterface $response, callable $next=null) {

        $rmnHeader = $request->getHeader('X-RMN-Lab');

        if (!count($rmnHeader)) {

            $message = json_encode([
                "code" => 1,
                "message" => 'Missing "X-RMN-Lab" header'
            ]);

            $response = $response->withStatus(400);
            $response = $response->withHeader('Content-Type', 'application/json');
            $response->getBody()->write($message);
            return $response;
        }

        preg_match('#rmn_request_(\d+)#', $rmnHeader[0], $matchesRequest);

        if (!count($matchesRequest)) {

            $message = json_encode([
                "code" => 2,
                "message" => 'Invalid "X-RMN-Lab" header: '.$rmnHeader[0]
            ]);

            $response = $response->withStatus(400);
            $response = $response->withHeader('Content-Type', 'application/json');
            $response->getBody()->write($message);
            return $response;
        }

        if (is_callable($next)) {

            $response = $next($request, $response);
        }


        $response = $response->withHeader('X-RMN-Lab-Response', 'rmn_response_'.$matchesRequest[1]);

        return $response;
    }
}