<?php
/**
 * Created by PhpStorm.
 * User: Noa
 * Date: 10/10/2017
 * Time: 13:28
 */

namespace Retailmenot\Lab\Middleware\middlewares\psr15;


use GuzzleHttp\Psr7\Response;
use Interop\Http\Server\MiddlewareInterface;
use Interop\Http\Server\RequestHandlerInterface;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

class RmnMiddlewarePsr15 implements MiddlewareInterface
{
    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler)
    {
        $response = new Response();

        $rmnHeader = $request->getHeader('X-RMN-Lab');

        if (!count($rmnHeader)) {

            $message = json_encode([
                "code" => 1,
                "message" => 'Missing "X-RMN-Lab" header'
            ]);

            $response = $response->withStatus(400);
            $response = $response->withHeader('Content-Type', 'application/json');
            $response->getBody()->write($message);
            return $response;
        }

        preg_match('#rmn_request_(\d+)#', $rmnHeader[0], $matchesRequest);

        if (!count($matchesRequest)) {

            $message = json_encode([
                "code" => 2,
                "message" => 'Invalid "X-RMN-Lab" header: '.$rmnHeader[0]
            ]);

            $response = $response->withStatus(400);
            $response = $response->withHeader('Content-Type', 'application/json');
            $response->getBody()->write($message);
            return $response;
        }

        $response = $handler->handle($request);

        $response = $response->withHeader('X-RMN-Lab-Response', 'rmn_response_'.$matchesRequest[1]);

        return $response;
    }
}